/**
	*****************************************************************************
	* @file		init.c
	* @author		Kenn Sebesta
	* @version	V3.1.2
	* @date		2011/03
	* @brief		This file contains functions for the PS3 Move's initialization
	*					sequences.
	*****************************************************************************
	* @copy
	*
	* <h2><center>&copy; COPYRIGHT 2011 Kenn Sebesta</center></h2>
	*/

#include "stm32f10x.h"
#include "init.h"
#include "defines.h"
#include "I2CRoutines.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
GPIO_InitTypeDef GPIO_InitStructure;

static const uint32_t ADC_CHANNEL[PS3_ADC_NUM_PINS] = PS3_ADC_CHANNELS;
static const uint32_t ADC_CHANNEL_MAPPING[PS3_ADC_NUM_PINS] = PS3_ADC_CHANNEL_MAPPING;
static ADC_TypeDef *ADC_MAPPING[PS3_ADC_NUM_PINS] = PS3_ADC_MAPPING;

/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);

/* Private functions ---------------------------------------------------------*/

/**
	* @brief	pinsInit program.
	* @param	None
	* @retval None
	*/
void pinsInit(){
	GPIO_StructInit(&GPIO_InitStructure); /* Reset init structure */

	/* Turn on all GPIO ports ***********************************************/
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |
								  RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
								  RCC_APB2Periph_GPIOE, ENABLE);
	
		/* Configure all unused GPIO port pins in Analog Input mode (floating
			input trigger OFF), this will reduce the power consumption and increase
			the device immunity against EMI/EMC **********************************/

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_Init(GPIOC, &GPIO_InitStructure);
	GPIO_Init(GPIOD, &GPIO_InitStructure);
	GPIO_Init(GPIOE, &GPIO_InitStructure);
}

/**
	* @brief	magDataPinInit program.
	* @param	None
	* @retval None
	*/
void magDataPinInit(){
	GPIO_StructInit(&GPIO_InitStructure); /* Reset init structure */

	/* Configure the  Magnetometer Data ready pin to floating input. */
	GPIO_InitStructure.GPIO_Pin = MAG_DATA_RDY_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(MAG_DATA_RDY_PORT, &GPIO_InitStructure);
}


/**
	* @brief	i2cInit program.
	* @param	None
	* @retval None
	*/
void i2cInit(){
	I2C_InitTypeDef  I2C_InitStructure;
	
	GPIO_StructInit(&GPIO_InitStructure); /* Reset init structure */

	/* I2C2 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);
	
	/* I2C2 SCK configuration to output open-drain */
	GPIO_InitStructure.GPIO_Pin = I2C2_SCK_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(I2C2_SCK_PORT, &GPIO_InitStructure);

	/* I2C2 SDA configuration to output open-drain  */
	GPIO_InitStructure.GPIO_Pin = I2C2_SDA_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(I2C2_SDA_PORT, &GPIO_InitStructure);

	/* Enable I2C2 reset state */
	RCC_APB1PeriphResetCmd(RCC_APB1Periph_I2C2, ENABLE);

	/* Release I2C2 from reset state */
	RCC_APB1PeriphResetCmd(RCC_APB1Periph_I2C2, DISABLE);

	/* I2C2 configuration, 400kHz, ACK enable */
	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_ClockSpeed = I2C2_CLOCK_SPEED;
	I2C_Init(I2C2, &I2C_InitStructure);
	
}

/**
	* @brief	adcInit program.
	* @param	None
	* @retval None
	*/
void adcInitSTM(){

	ADC_InitTypeDef  ADC_InitStructure;
  /* PCLK2 is the APB2 clock */
  /* ADCCLK = PCLK2/6 = 72/6 = 12MHz*/
  RCC_ADCCLKConfig(RCC_PCLK2_Div6);

  /* Enable ADC1 clock so that we can talk to it */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
  /* Put everything back to power-on defaults */
  ADC_DeInit(ADC1);

  /* ADC1 Configuration ------------------------------------------------------*/
  /* ADC1 and ADC2 operate independantly */
  ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
  /* Disable the scan conversion so we do one at a time */
  ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  /* Don't do contimuous conversions - do them on demand */
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  /* Start conversion by software, not an external trigger */
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
  /* Conversions are 12 bit - put them in the lower 12 bits of the result */
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  /* Say how many channels will be used by the sequencer */
  ADC_InitStructure.ADC_NbrOfChannel = 1;

  /* Now do the setup */
  ADC_Init(ADC1, &ADC_InitStructure);
  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);

  /* Enable ADC1 reset calibaration register */
  ADC_ResetCalibration(ADC1);
  /* Check the end of ADC1 reset calibration register */
  while(ADC_GetResetCalibrationStatus(ADC1));
  /* Start ADC1 calibration */
  ADC_StartCalibration(ADC1);
  /* Check the end of ADC1 calibration */
  while(ADC_GetCalibrationStatus(ADC1));

}

/**
	* @brief	adcInit program.
	* @param	None
	* @retval None
	*/
void adcInitPIOS(){
	ADC_DeInit(ADC1);
	ADC_DeInit(ADC2);

	/* Enable ADC clocks */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_ADC2, ENABLE);
	
	/* Map channels to conversion slots depending on the channel selection mask */
	for (int32_t i = 0; i < PS3_ADC_NUM_PINS; i++) {
		ADC_RegularChannelConfig(ADC_MAPPING[i], ADC_CHANNEL[i],
					 ADC_CHANNEL_MAPPING[i],
					 PS3_ADC_ADCCLK);
	}
	
	/* Configure ADCs */
	ADC_InitTypeDef ADC_InitStructure;
	ADC_StructInit(&ADC_InitStructure);
	ADC_InitStructure.ADC_Mode = ADC_Mode_RegSimult;
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = ((PS3_ADC_NUM_CHANNELS + 1) >> 1);
	ADC_Init(ADC1, &ADC_InitStructure);
	
	ADC_Init(ADC2, &ADC_InitStructure);
	
	/* Enable ADC2 external trigger conversion (to synch with ADC1) */
	ADC_ExternalTrigConvCmd(ADC2, ENABLE);

	/* Configure the ADC clock to an 8 times prescaler */	
	RCC_ADCCLKConfig(RCC_PCLK2_Div8);
	
	/* ADC1 calibration */
	ADC_Cmd(ADC1, ENABLE);
	ADC_ResetCalibration(ADC1);
	while (ADC_GetResetCalibrationStatus(ADC1)) ;
	ADC_StartCalibration(ADC1);
	while (ADC_GetCalibrationStatus(ADC1)) ;
	
	/* ADC2 calibration */
	ADC_Cmd(ADC2, ENABLE);
	ADC_ResetCalibration(ADC2);
	while (ADC_GetResetCalibrationStatus(ADC2)) ;
	ADC_StartCalibration(ADC2);
	while (ADC_GetCalibrationStatus(ADC2)) ;
	
	/* Finally start initial conversion */
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);
	ADC_SoftwareStartConvCmd(ADC2, ENABLE);
}


/**
	* @brief	analogInit program.
	* @param	None
	* @retval None
	*/
void analogPinsInit(){
	GPIO_StructInit(&GPIO_InitStructure); /* Reset init structure */

	/* Configure the ACC_X input pin as analog input */
	GPIO_InitStructure.GPIO_Pin = ACC_X_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	
	GPIO_Init(ACC_X_PORT, &GPIO_InitStructure);

	/* Configure the ACC_Y input pin as analog input */
	GPIO_InitStructure.GPIO_Pin = ACC_Y_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	
	GPIO_Init(ACC_Y_PORT, &GPIO_InitStructure);

	/* Configure the ACC_Z input pin as analog input */
	GPIO_InitStructure.GPIO_Pin = ACC_Z_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	
	GPIO_Init(ACC_Z_PORT, &GPIO_InitStructure);

	/* Configure the GYRO_X input pin as analog input */
	GPIO_InitStructure.GPIO_Pin = GYRO_X_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	
	GPIO_Init(GYRO_X_PORT, &GPIO_InitStructure);

	/* Configure the GYRO_Y input pin as analog input */
	GPIO_InitStructure.GPIO_Pin = GYRO_Y_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	
	GPIO_Init(GYRO_Y_PORT, &GPIO_InitStructure);

	/* Configure the GYRO_Z input pin as analog input */
	GPIO_InitStructure.GPIO_Pin = GYRO_Z_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	
	GPIO_Init(GYRO_Z_PORT, &GPIO_InitStructure);

	/* Configure the TRIGGER_Z input pin as analog input */
	GPIO_InitStructure.GPIO_Pin = TRIGGER_Z_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	
	GPIO_Init(TRIGGER_Z_PORT, &GPIO_InitStructure);

}


/**
	* @brief	pwrEnableInit program.
	* @param	None
	* @retval None
	*/
void pwrEnableInit(){
	GPIO_StructInit(&GPIO_InitStructure); /* Reset init structure */

	/* Configure the `Enable` pin as output push-pull */
	GPIO_InitStructure.GPIO_Pin = PWR_ENABLE_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(PWR_ENABLE_PORT, &GPIO_InitStructure);
}


/**
	* @brief	ledsInit program.
	* @param	None
	* @retval None
	*/
void ledsInit(){
	GPIO_StructInit(&GPIO_InitStructure); /* Reset init structure */

	/* Configure the `Red` LED pin as output push-pull */
	GPIO_InitStructure.GPIO_Pin = LED_R_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(LED_R_PORT, &GPIO_InitStructure);

	LED_R_PORT->BRR=LED_R_PIN;		//LED off

	/* Configure the `Green` LED pin as output push-pull */
	GPIO_InitStructure.GPIO_Pin = LED_G_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(LED_G_PORT, &GPIO_InitStructure);

	LED_G_PORT->BRR=LED_G_PIN;		//LED off

	/* Configure the `Blue` LED pin as output push-pull */
	GPIO_InitStructure.GPIO_Pin = LED_B_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(LED_B_PORT, &GPIO_InitStructure);

	LED_B_PORT->BRR=LED_B_PIN;		//LED off
}


/**
	* @brief	buttonsInit program.
	* @param	None
	* @retval None
	*/
void buttonsInit(){
	GPIO_StructInit(&GPIO_InitStructure); /* Reset init structure */

	/* Configure the `PS` Button pin as floating input*/
	GPIO_InitStructure.GPIO_Pin = BUTTON_PS_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_PS_PORT, &GPIO_InitStructure);

	
	/* Configure the `Select` Button pin as pull-down */
	GPIO_InitStructure.GPIO_Pin = BUTTON_SELECT_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_SELECT_PORT, &GPIO_InitStructure);

	
	/* Configure the `Start` Button pin as pull-down */
	GPIO_InitStructure.GPIO_Pin = BUTTON_START_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_START_PORT, &GPIO_InitStructure);

	
	/* Configure the `Circle` Button pin as pull-down */
	GPIO_InitStructure.GPIO_Pin = BUTTON_CIRCLE_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_CIRCLE_PORT, &GPIO_InitStructure);


	/* Configure the `X` Button pin as pull-down */
	GPIO_InitStructure.GPIO_Pin = BUTTON_X_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_X_PORT, &GPIO_InitStructure);


	/* Configure the `Triangle` Button pin as pull-down */
	GPIO_InitStructure.GPIO_Pin = BUTTON_TRIANGLE_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_TRIANGLE_PORT, &GPIO_InitStructure);


	/* Configure the `Square` Button pin as pull-down */
	GPIO_InitStructure.GPIO_Pin = BUTTON_SQUARE_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_SQUARE_PORT, &GPIO_InitStructure);


	/* Configure the `Move` Button pin as pull-down */
	GPIO_InitStructure.GPIO_Pin = BUTTON_MOVE_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(BUTTON_MOVE_PORT, &GPIO_InitStructure);
}
